%code requires {
  #include <inttypes.h>

  typedef struct node_t node_t;
}

%code top {
  #include <stdio.h>
  #include <stdlib.h>

  #include "calc.h"
  #include "error.h"

  void yyerror (node_t **locp, char const *msg);
  int yylex (void);
}

%parse-param {node_t **node_root}
%union {
  node_t *node;
  int64_t value;
}

%token <value> NUMBER
%token PLUS MINUS TIMES DIVIDE POWER LPARENTHESIS RPARENTHESIS;
%left PLUS MINUS;
%left TIMES DIVIDE;
%precedence NEG;
%right POWER;

%type <node> calculation expression factor purefactor
%start calculation;

%code {

  node_t *node_new (node_kind kind, node_t *node_l, node_t *node_r)
  {
    static uint8_t node_count = 0;

    node_t *node = (node_t*) malloc(sizeof(node_t));

    node->kind = kind;
    node->content.binop.id = node_count++;
    node->content.binop.node_l = node_l;
    node->content.binop.node_r = node_r;

    return node;
  }

  node_t *node_new_number (int64_t value)
  {
    node_t *node = (node_t*) malloc(sizeof(node_t));

    node->kind = NODE_NUMBER;
    node->content.value = value;

    return node;
  }
}
%%

calculation : expression { 
  $$ = $1;
  *node_root = $$;
}
| %empty {
  $$ = node_new_number(0);
  *node_root = $$;
}

expression  : expression PLUS expression    { $$ = node_new(NODE_PLUS, $1, $3); }
            | expression MINUS expression   { $$ = node_new(NODE_MINUS, $1, $3); }
            | expression TIMES expression   { $$ = node_new(NODE_TIMES, $1, $3); }
            | expression DIVIDE expression  { $$ = node_new(NODE_DIVIDE, $1, $3); }
            | MINUS expression %prec NEG    { $$ = node_new(NODE_MINUS, node_new_number(0), $2); }
            | expression POWER expression   { $$ = node_new(NODE_POWER, $1, $3); }
            | factor                        { $$ = $1; }
            ;

factor  : NUMBER            { $$ = node_new_number($1); }
        | NUMBER purefactor { $$ = node_new(NODE_TIMES, node_new_number($1), $2); }
        | purefactor        { $$ = $1; }
        ;

purefactor  : LPARENTHESIS expression RPARENTHESIS            { $$ = $2; }
            | LPARENTHESIS expression RPARENTHESIS purefactor { $$ = node_new(NODE_TIMES, $2, $4); }
            ;
%%

void yyerror (node_t **locp, char const *msg)
{
  fprintf(stderr, "Parsing Error: %s.\n", msg);
  error(ERROR_PARSING);
}
